@@ -186,12 +186,12 @@ class LinearAttention(nn.Module):
         
         if causal:
             kv = torch.cumsum(torch.matmul(k.unsqueeze(-1), v.unsqueeze(-2)), dim=1)
-            k1 = torch.cumsum(k, dim=1)
+            kn = torch.cumsum(k, dim=1)
         else:
             kv = torch.einsum('zsD, zsd -> zDd', k, v).unsqueeze(1)
-            k1 = k.sum(dim=1, keepdim=True)
+            kn = k.sum(dim=1, keepdim=True)
         
-        out = torch.matmul(q.unsqueeze(-2), kv).squeeze(-2) / (q*k1).sum(-1, keepdim=True)
+        out = torch.matmul(q.unsqueeze(-2), kv).squeeze(-2) / torch.matmul(q.unsqueeze(-2), kn.unsqueeze(-1)).squeeze(-1)
         out = rearrange(out, '(b h) s d -> b s (h d)', h=self.n_heads)
         return self.out_proj(out)
 
@@ -269,7 +269,7 @@ class OrthoLinearAttention(nn.Module):
             kv = torch.einsum('zsD, zsd -> zDd', k, v).unsqueeze(1)
             kn = k.sum(1, keepdim=True)
         
-        out = torch.matmul(q.unsqueeze(-2), kv).squeeze(-2) / (q * kn).sum(-1, keepdim=True)
+        out = torch.matmul(q.unsqueeze(-2), kv).squeeze(-2) / torch.matmul(q.unsqueeze(-2), kn.unsqueeze(-1)).squeeze(-1)
         out = rearrange(out, '(b h) s d -> b s (h d)', h=self.n_heads)
         return self.out_proj(out)
 
@@ -597,7 +597,6 @@ class SlidingWindowAttention(nn.Module):
                 pad = (pad_amount, pad_amount)
 
             q = dilated_sliding_window(q, size=src_len, dim=1, stride=self.dilation, dilation=1, pad=pad)
-            # q = dilated_sliding_window_nopad(q, size=src_len, dim=1, stride=self.dilation, dilation=1, pad=pad)
 
             attn_output_weights = torch.einsum('zbsd, zsd -> zbs', q, k)  # (bsz * n_heads, n_bands, src_len)
             
@@ -631,3 +630,4 @@ class SlidingWindowAttention(nn.Module):
         if self.batch_first:
             return attn_output.transpose(0, 1)
         return attn_output
+
